import greenfoot.Actor;

/**
 * Implementing a single cell. Each cell has its own live or death state information
 * and a reference to an array containing its surrounding neighbours,
 *
 */
public class Cell extends Actor {

   CellState currentState, previousState;
   Cell[] neighbours; // Will be initialized in by LifeWorld.getCellsNeighbours

   private void setState(final CellState newState) {
      currentState = newState;
      setImage(currentState.image);
   }
   
   /**
    * Setting the current cell's array of neighbours.
    * 
    * @param neighbors The cell's set of neighbours  
    */
   public void setNeighbors(Cell[] neighbors) {
      this.neighbours = neighbors;
   }

   /**
    * Create the cell body's image and initialize state.
    * 
    * @param intialState Indicating {@link CellState#ALIVE} or {@link CellState#DEAD} 
    */
   public Cell(final CellState intialState) {
      super();
      setState(intialState);
      saveCurrentToPreviousState();
   }

   /**
    * Count neighbouring cells currently living based on the result
    * of {@link #setNeighbors(Cell[])}. Then apply Game of Life
    * rules and show cell state.
    */
   public void act() {
      final int livingNeighboursCount = getLivingNeighbourCount();
      switch(previousState) {

      case ALIVE:
         if (livingNeighboursCount < 2 ||        // Dying on account of under-population
               3 < livingNeighboursCount) {      // Dying on account of over-population
            setState(CellState.DEAD);
         } 
         break;

      case DEAD:
         if (3 == livingNeighboursCount) {     // Born by reproduction
            setState(CellState.ALIVE);
         }
         break;
      }
   }

   /**
    * Save the previous state in preparation for the next time step.
    */
   public void saveCurrentToPreviousState() {
      previousState = currentState;
   }

   /**
    * @return Number of living neighbouring cells
    */
   public int getLivingNeighbourCount() {
      int sum = 0;
      for (final Cell neighbor: neighbours) {
         sum += neighbor.previousState.liveCount;
      }
      return sum;
   }
   
   /**
    * Set cell alive
    */
   public void setAlive() {
      setState(CellState.ALIVE);
   }
}
